/**************************************************************************
   STV-TOOLS: Shannon Total Variation based image processing
   Copyright (C) 2016 Remy Abergel (remy.abergel AT gmail.com),
   Lionel Moisan (Lionel.Moisan AT parisdescartes.fr).

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/

/*

  Compilation (with gcc): gcc -O3 tiffprintasc.c tiffread.c -lm -ltiff -o tiffprintasc

  References :

  [1] R. Abergel and L. Moisan, ``The Shannon Total Variation'', preprint MAP5 n° 2016-19, July 2016.
  [2] L. Moisan. How to discretize the total variation of an image? Proceedings in Applied Mathematics and Mechanics, 2007.

  Versions :

  + v 1.0 : (09/2016) initial version (Rémy Abergel).

 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#define VERSION "1.0"

#define FREE_ALL() {				\
    if(NULL != u) free(u);			\
  }
#define ASSERT_ALLOC(cmd) if(NULL == (cmd)) { printf("Not enough memory\n"); FREE_ALL(); exit(EXIT_FAILURE); }

static void display_usage()
{
  printf("\nPrint the graylevels of a TIFF image (row major order) in ascii format.\n\n");
  printf("Usage: tiffprintasc in\n\n");
  printf("   in        : intput image\n\n");
}


int main(int argc, char **argv)
{
  char *filename_in = NULL;
  double *u;
  int i,adr,nx,ny;

  /******************************************/
  /*            arguments parser            */
  /******************************************/

  /* main parser */
  for(i=1;i<argc;i++) {
    if (NULL == filename_in) filename_in = argv[i];
    else { display_usage(); printf("Error: could not retrieve properly input arguments.\n\n"); return EXIT_FAILURE; }
  }

  /* check consistency */
  if (NULL == filename_in) {
    display_usage();
    printf("Error: input 'in' is missing\n\n");
    return EXIT_FAILURE;
  }


  /************************************************************/
  /* Open TIFF Image & get its graylevels in double precision */
  /************************************************************/
  if(EXIT_FAILURE == read_tiff_into_double(&u,&nx,&ny,filename_in,0)) {
    printf("An error occured during the image reading process.\n");
    printf("Free memory and exit the program.\n\n");
    FREE_ALL();
    return EXIT_FAILURE;
  }

  /************************************************************/
  /*                kernel : printf graylevels                */
  /************************************************************/
  for(adr=0;adr<nx*ny;adr++) printf("%.17g\n",u[adr]);

  /* free memory */
  FREE_ALL();

}
